#include "marchingCube.h"
#include "mti.h"

#define TYPE_C 0
#define TYPE_I 1

int d_type=TYPE_C;
int drawtype = 5;	//draw mode default(0:cloud 1:iso-surface-cloud 2:point 3:iso-surface 4:wireframe 5:non)
int draw_mode = 5;

int ISO_MAX=40;
int ISO_MIN=25;
int alpha_on=1;
float maxv,minv;

class CAtom{
public:
	int    number;
	double charge;
	ogVector position;
	void loadCube( FILE *fp ){ fscanf( fp, "%d%lf%lf%lf%lf", &number, &charge, &position.x, &position.y, &position.z ); }
	void saveCube( FILE *fp ){ fprintf( fp, "%d %f %f %f %f\n", number, float(charge), float(position.x), float(position.y), float(position.z) ); }
	void loadBin( FILE *fp ){
		float c, x, y, z;
		fread( (int   *)&number, 1, sizeof(int), fp );
		fread( (float *)&c, 1, sizeof(float), fp );
		fread( (float *)&x, 1, sizeof(float), fp );
		fread( (float *)&y, 1, sizeof(float), fp );
		fread( (float *)&z, 1, sizeof(float), fp );
		charge = c; position.x = x; position.y = y; position.z = z;
	}
	void saveBin( FILE *fp ){
		float c = charge; float x = position.x; float y = position.y; float z = position.z;
		fwrite( (int   *)&number, 1, sizeof(int), fp );
		fwrite( (float *)&c, 1, sizeof(float), fp );
		fwrite( (float *)&x, 1, sizeof(float), fp );
		fwrite( (float *)&y, 1, sizeof(float), fp );
		fwrite( (float *)&z, 1, sizeof(float), fp );
	}
};
//-------------------------------------------------------------------------------------------
class CCube{
public:
	int     IFlag;
	ogVector origin;
	int     xSize;
	ogVector xUnit;
	int     ySize;
	ogVector yUnit;
	int     zSize;
	ogVector zUnit;
	ogVector center;
	CAtom  *atom;
	int     num1;
	int     num2;
	float  *d;
	GLuint *texName;
	char filename0[256]; 

	float density( int x, int y, int z ){ return d[z + (y + x*ySize) * zSize]; }
	void convert(){ 
		float maxd;
		if(maxv>fabsf(minv))maxd=maxv;
		else maxd=fabsf(minv);
		for( long i=0; i<xSize*ySize*zSize; i++ ) d[i] = d[i]*(255.0/maxd);
	}
	void convert( double k1, double k2 ){
		for( long i=0; i<xSize*ySize*zSize; i++ ) d[i] = (d[i] > 0 ? 1.0 : -1.0) * k1 * exp( k2 * log( fabsf(d[i]) ) );
	}
	void construct(){ atom = NULL; d = NULL; texName = NULL; }
//	void construct( char *filename, double k1, double k2 ){ this->construct(); this->load( filename ); this->convert( k1, k2 ); }
	int construct( char *filename, double k1, double k2 ){ int r=0; this->construct(); r = this->load( filename ); this->convert( ); return r;}
	CCube(){ this->construct(); }
	CCube( char *filename, double k1, double k2 ){ this->construct( filename, k1, k2 ); }
	~CCube(){ if( atom != NULL ) delete atom; if( d != NULL ) delete d; if( texName != NULL ) delete texName; }
	void glTexImage( ogImage &image, int textureNumber );
	void texImage();
	GLubyte NormalValue(int x, int y, int z, float densityf);


	void drawYZ( int x );
	void drawZX( int y );
	void drawXY( int z );
	void drawYZ(){ for( int x=0; x<xSize; x++ ){ this->drawYZ( x ); }}
	void drawZY(){ for( int x=xSize-1; x>=0; x-- ){ this->drawYZ( x ); }}
	void drawZX(){ for( int y=0; y<ySize; y++ ){ this->drawZX( y ); }}
	void drawXZ(){ for( int y=ySize-1; y>=0; y-- ){ this->drawZX( y ); }}
	void drawXY(){ for( int z=0; z<zSize; z++ ){ this->drawXY( z ); }}
	void drawYX(){ for( int z=zSize-1; z>=0; z-- ){ this->drawXY( z ); }}

	void drawYZ( int x, double intensity, double ratio, CCube &cube );
	void drawZX( int y, double intensity, double ratio, CCube &cube );
	void drawXY( int z, double intensity, double ratio, CCube &cube );
	void drawYZ( double intensity, double ratio, CCube &cube ){ for( int x=0; x<xSize; x++ ) this->drawYZ( x, intensity, ratio, cube ); }
	void drawZY( double intensity, double ratio, CCube &cube ){ for( int x=xSize-1; x>=0; x-- ) this->drawYZ( x, intensity, ratio, cube ); }
	void drawZX( double intensity, double ratio, CCube &cube ){ for( int y=0; y<ySize; y++ ) this->drawZX( y, intensity, ratio, cube ); }
	void drawXZ( double intensity, double ratio, CCube &cube ){ for( int y=ySize-1; y>=0; y-- ) this->drawZX( y, intensity, ratio, cube ); }
	void drawXY( double intensity, double ratio, CCube &cube ){ for( int z=0; z<zSize; z++ ) this->drawXY( z, intensity, ratio, cube ); }
	void drawYX( double intensity, double ratio, CCube &cube ){ for( int z=zSize-1; z>=0; z-- ) this->drawXY( z, intensity, ratio, cube ); }
	void draw( ogVector &direction, double intensity );
	void draw( ogVector &direction, double intensity, double ratio, CCube &cube );

	void drawAtoms();
	void drawAtoms( double ratio, CCube &cube );
	void drawBonds();
	void drawBonds( double ratio, CCube &cube );
	void drawFrame( GLubyte red, GLubyte green, GLubyte blue );

	int loadCube( char *filename );
	int saveCube( char *filename );
	int loadBin( char *filename );
	int saveBin( char *filename );
	int load( char *filename );
	int save( char *filename );


	void interpolate( CCube &cube1, CCube &cube2, double a0, double b0 ){
		double a = a0 / (a0+b0);
		double b = b0 / (a0+b0);
		for( long i=0; i<xSize*ySize*zSize; i++ ) d[i] = cube1.d[i] * a + cube2.d[i] * b;
	}
};

GLubyte CCube::NormalValue(int x, int y, int z, float densityf){
	ogVector Light;
	ogVector sv[6];
	ogVector svd;
	ogVector normal;
	GLubyte d[7];
	double cosValue;

	if(x==0 || y==0 || z==0 || x==xSize-1 || y==ySize-1 || z==zSize-1) return 255;

	Light.set(1.0, 1.0, 1.0);
	Light.normalize();

	sv[0].set(-1.0, 0.0, 0.0);
	sv[1].set( 1.0, 0.0, 0.0);
	sv[2].set( 0.0,-1.0, 0.0);
	sv[3].set( 0.0, 1.0, 0.0);
	sv[4].set( 0.0, 0.0,-1.0);
	sv[5].set( 0.0, 0.0, 1.0);

	if(densityf>=0){
		d[0] = cramp( int(this->density( x-1,   y,   z )) );
		d[1] = cramp( int(this->density( x+1,   y,   z )) );
		d[2] = cramp( int(this->density(   x, y-1,   z )) );
		d[3] = cramp( int(this->density(   x, y+1,   z )) );
		d[4] = cramp( int(this->density(   x,   y, z-1 )) );
		d[5] = cramp( int(this->density(   x,   y, z+1 )) );
		d[6] = cramp( int(this->density( x, y, z )) );
	}
	else{
		d[0] = cramp( int(-this->density( x-1,   y,   z )) );
		d[1] = cramp( int(-this->density( x+1,   y,   z )) );
		d[2] = cramp( int(-this->density(   x, y-1,   z )) );
		d[3] = cramp( int(-this->density(   x, y+1,   z )) );
		d[4] = cramp( int(-this->density(   x,   y, z-1 )) );
		d[5] = cramp( int(-this->density(   x,   y, z+1 )) );
		d[6] = cramp( int(-this->density( x, y, z )) );
	}
	normal.set(0.0,0.0,0.0);
	for(int i=0;i<6;i++){
		svd = sv[i]*(double)(d[6]-d[i]);
		normal+=svd;
	}
	normal.normalize();
	cosValue=Light*normal;
	//-1 < Light*normal < 1  :  1 - 255
	cosValue=127*cosValue+128;
	//Ambient Light : 60
	return GLubyte((195.0/255.0)*cosValue+60);
}

void CCube::glTexImage( ogImage &image, int textureNumber ){
	glGenTextures( 1, &texName[textureNumber] );
	glBindTexture( GL_TEXTURE_2D, texName[textureNumber] );
	glPixelStorei( GL_UNPACK_ALIGNMENT,1 );
	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
	glTexImage2D( GL_TEXTURE_2D, 0, image.format(), image.width, image.height, 0, image.format(), GL_UNSIGNED_BYTE, image.array );
}

void CCube::texImage(){
	if( texName != NULL ) return;
	texName = new GLuint[xSize+ySize+zSize];
	int imin,imax;

	imin=ISO_MAX;
	imax=imin+15;
	if(imax>255)imax=255;

/*
	FILE *fp;
	if(d_type==TYPE_C){
		char filename_c[256];
		strcpy(filename_c,filename0);
		mitfn_type(filename_c, 'c');
//		fp = mti_fopen(filename_c,"wb");
		fp = mti_fopen(filename_c,"rb");
	}
	else{
		char filename_i[256];
		strcpy(filename_i,filename0);
		mitfn_type(filename_i, 'i');
//		fp = mti_fopen(filename_i,"wb");
		fp = mti_fopen(filename_i,"rb");
	}

	ogImage image128( 128, 128, 4 );
	for( int x=0; x<xSize; x++ ){
		for(int i=0;i<128*128*4;i++)image128.array[i]=mit_readb(fp);
		this->glTexImage( image128, x );
	}
	for( int y=0; y<ySize; y++ ){
		for(int i=0;i<128*128*4;i++)image128.array[i]=mit_readb(fp);
		this->glTexImage( image128, xSize + y );
	}
	for( int z=0; z<zSize; z++ ){
		for(int i=0;i<128*128*4;i++)image128.array[i]=mit_readb(fp);
		this->glTexImage( image128, xSize + ySize + z );
	}
*/
	ogImage image( ySize, zSize, 4 );
	ogImage image128( 128, 128, 4 );
	float densityf;
	int densityi;
	for( int x=0; x<xSize; x++ ){
		for( int z=0; z<zSize; z++ ){
		for( int y=0; y<ySize; y++ ){
			densityf = this->density( x, y, z );
			densityf > 0 ? densityi = cramp(int(densityf)) : densityi = cramp(int(-densityf));
			if(d_type==TYPE_C){
				if( densityf > 0 ) image.set( y, z, 255, 255, 255, densityi );
				else image.set( y, z, 255, 0, 0, densityi );
			}
			else{
				GLubyte n = this->NormalValue(x,y,z,densityf);
				if(imin<=densityi && densityi<=imax){
					densityf > 0 ? image.set( y, z, n, n, n, 255 ) : image.set( y, z, n, 0, 0, 255 );
				}
				else{
					densityf > 0 ? image.set( y, z, n, n, n, 0 ) : image.set( y, z, n, 0, 0, 0 );
				}
			}
		}}

		gluScaleImage( GL_RGBA, ySize, zSize, GL_UNSIGNED_BYTE, image.array, 128, 128, GL_UNSIGNED_BYTE, image128.array );
		this->glTexImage( image128, x );
	}
	image.construct( zSize, xSize, 4 );
	for( int y=0; y<ySize; y++ ){
		for( int x=0; x<xSize; x++ ){
		for( int z=0; z<zSize; z++ ){
			densityf = this->density( x, y, z );
			densityf > 0 ? densityi = cramp(int(densityf)) : densityi = cramp(int(-densityf));
			if(d_type==TYPE_C){
				if( densityf > 0 ) image.set( z, x, 255, 255, 255, densityi );
				else image.set( z, x, 255, 0, 0, densityi );
			}
			else{
				GLubyte n = this->NormalValue(x,y,z,densityf);
				if(imin<=densityi && densityi<=imax){
					densityf > 0 ? image.set( z, x, n, n, n, 255 ) : image.set( z, x, n, 0, 0, 255 );
				}
				else{
					densityf > 0 ? image.set( z, x, n, n, n, 0 ) : image.set( z, x, n, 0, 0, 0 );
				}
			}
		}}

		gluScaleImage( GL_RGBA, zSize, xSize, GL_UNSIGNED_BYTE, image.array, 128, 128, GL_UNSIGNED_BYTE, image128.array );
		this->glTexImage( image128, xSize + y );
	}
	image.construct( xSize, ySize, 4 );
	for( int z=0; z<zSize; z++ ){
		for( int y=0; y<ySize; y++ ){
		for( int x=0; x<xSize; x++ ){
			densityf = this->density( x, y, z );
			densityf > 0 ? densityi = cramp(int(densityf)) : densityi = cramp(int(-densityf));
			if(d_type==TYPE_C){
				if( densityf > 0 ) image.set( x, y, 255, 255, 255, densityi );
				else image.set( x, y, 255, 0, 0, densityi );
			}
			else{
				GLubyte n = this->NormalValue(x,y,z,densityf);
				if(imin<=densityi && densityi<=imax){
					densityf > 0 ? image.set( x, y, n, n, n, 255 ) : image.set( x, y, n, 0, 0, 255 );
				}
				else{
					densityf > 0 ? image.set( x, y, n, n, n, 0 ) : image.set( x, y, n, 0, 0, 0 );
				}
			}
		}}

		gluScaleImage( GL_RGBA, xSize, ySize, GL_UNSIGNED_BYTE, image.array, 128, 128, GL_UNSIGNED_BYTE, image128.array );
		this->glTexImage( image128, xSize + ySize + z );
	}

/*	mti_fclose(fp);*/
//	if( d != NULL ) delete d;
//	d = NULL;
}

void CCube::drawYZ( int x ){
	glBindTexture( GL_TEXTURE_2D, texName[x] );
	glBegin(GL_POLYGON);
	glTexCoord2i( 0, 0 );
	glVertex3d( double(x) * xUnit.abs(), 0, 0 );
	glTexCoord2i( 0, 1 );
	glVertex3d( double(x) * xUnit.abs(), 0, zSize * zUnit.abs() );
	glTexCoord2i( 1, 1 );
	glVertex3d( double(x) * xUnit.abs(), ySize * yUnit.abs(), zSize * zUnit.abs() );
	glTexCoord2i( 1, 0 );
	glVertex3d( double(x) * xUnit.abs(), ySize * yUnit.abs(), 0 );
	glEnd();
}

void CCube::drawZX( int y ){
	glBindTexture( GL_TEXTURE_2D, texName[xSize+y] );
	glBegin(GL_POLYGON);
	glTexCoord2i( 0, 0 );
	glVertex3d( 0, double( y ) * yUnit.abs(), 0 );
	glTexCoord2i( 0, 1 );
	glVertex3d( xSize * xUnit.abs(), double( y ) * yUnit.abs(), 0 );
	glTexCoord2i( 1, 1 );
	glVertex3d( xSize * xUnit.abs(), double( y ) * yUnit.abs(), zSize * zUnit.abs());
	glTexCoord2i( 1, 0 );
	glVertex3d( 0, double( y ) * yUnit.abs(), zSize * zUnit.abs());
	glEnd();
}

void CCube::drawXY( int z ){
	glBindTexture( GL_TEXTURE_2D, texName[xSize+ySize+z] );
	glBegin(GL_POLYGON);
	glTexCoord2i( 0, 0 );
	glVertex3d( 0, 0, double( z ) * zUnit.abs());
	glTexCoord2i( 0, 1 );
	glVertex3d( 0, ySize * yUnit.abs(), double( z ) * zUnit.abs() );
	glTexCoord2i( 1, 1 );
	glVertex3d( xSize * xUnit.abs(), ySize * yUnit.abs(), double( z ) * zUnit.abs() );
	glTexCoord2i( 1, 0 );
	glVertex3d( xSize * xUnit.abs(), 0, double( z ) * zUnit.abs() );
	glEnd();
}

void CCube::drawYZ( int x, double intensity, double ratio, CCube &cube ){
	if(d_type==TYPE_C){
		glBlendFunc( GL_ZERO,GL_ONE_MINUS_SRC_ALPHA );
		glColor4ub( 255, 255, 255, cramp(int(255.0*intensity)) );
		ratio < 0.5 ? this->drawYZ( x ) : cube.drawYZ( x );
		glBlendFunc( GL_SRC_ALPHA,GL_ONE );
		glColor4ub( 255, 255, 255, cramp(int(255.0*intensity*(1.0-ratio))) ); this->drawYZ( x );
		glBlendFunc( GL_SRC_ALPHA,GL_ONE );
		glColor4ub( 255, 255, 255, cramp(int(255.0*intensity*ratio)) ); cube.drawYZ( x );
	}
	else{
		double alpha = 0.5/(1.0-0.5*ratio);
		double beta = 0.5*ratio;
		glBlendFunc( GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA );
		glColor4ub( 255, 255, 255, cramp(int(255.0*(1.0-alpha))) );
		this->drawYZ( x );
		glBlendFunc( GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA );
		glColor4ub( 255, 255, 255, cramp(int(255.0*beta)) );
		cube.drawYZ( x );
	}
}

void CCube::drawZX( int y, double intensity, double ratio, CCube &cube ){
	if(d_type==TYPE_C){
		glBlendFunc( GL_ZERO,GL_ONE_MINUS_SRC_ALPHA );
		glColor4ub( 255, 255, 255, cramp(int(255.0*intensity)) );
		ratio < 0.5 ? this->drawZX( y ) : cube.drawZX( y );
		glBlendFunc( GL_SRC_ALPHA,GL_ONE );
		glColor4ub( 255, 255, 255, cramp(int(255.0*intensity*(1.0-ratio))) ); this->drawZX( y );
		glBlendFunc( GL_SRC_ALPHA,GL_ONE );
		glColor4ub( 255, 255, 255, cramp(int(255.0*intensity*ratio)) ); cube.drawZX( y );
	}
	else{
		double alpha = 0.5/(1.0-0.5*ratio);
		double beta = 0.5*ratio;
		glBlendFunc( GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA );
		glColor4ub( 255, 255, 255, cramp(int(255.0*(1.0-alpha))) );
		this->drawZX( y );
		glBlendFunc( GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA );
		glColor4ub( 255, 255, 255, cramp(int(255.0*beta)) );
		cube.drawZX( y );
	}
}

void CCube::drawXY( int z, double intensity, double ratio, CCube &cube ){
	if(d_type==TYPE_C){
		glBlendFunc( GL_ZERO,GL_ONE_MINUS_SRC_ALPHA );
		glColor4ub( 255, 255, 255, cramp(int(255.0*intensity)) );
		ratio < 0.5 ? this->drawXY( z ) : cube.drawXY( z );
		glBlendFunc( GL_SRC_ALPHA,GL_ONE );
		glColor4ub( 255, 255, 255, cramp(int(255.0*intensity*(1.0-ratio))) ); this->drawXY( z );
		glBlendFunc( GL_SRC_ALPHA,GL_ONE );
		glColor4ub( 255, 255, 255, cramp(int(255.0*intensity*ratio)) ); cube.drawXY( z );
	}
	else{
		double alpha = 0.5/(1.0-0.5*ratio);
		double beta = 0.5*ratio;
		glBlendFunc( GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA );
		glColor4ub( 255, 255, 255, cramp(int(255.0*(1.0-alpha))) );
		this->drawXY( z );
		glBlendFunc( GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA );
		glColor4ub( 255, 255, 255, cramp(int(255.0*beta)) );
		cube.drawXY( z );
	}
}

void CCube::draw( ogVector &direction, double intensity ){
	if( texName == NULL ) this->texImage();
	glBlendFunc( GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA );
	glColor4ub( 255, 255, 255, cramp(int(255*intensity)) );
	og.PushMatrix();
	og.Translate(-center);
	if( fabs(direction.x) > fabs(direction.y) && fabs(direction.x) > fabs(direction.z) ){
		if( direction.x < 0 ) this->drawYZ();
		else                  this->drawZY();
	}else if( fabs(direction.y) > fabs(direction.z) ){
		if( direction.y < 0 ) this->drawZX();
		else                  this->drawXZ();
	}else{
		if( direction.z < 0 ) this->drawXY();
		else                  this->drawYX();
	}
	og.PopMatrix();
}

void CCube::draw( ogVector &direction, double intensity, double ratio, CCube &cube ){
	if( texName == NULL ) this->texImage();
	if( cube.texName == NULL ) cube.texImage();
	og.PushMatrix();
	og.Translate(-center);
	if( fabs(direction.x) > fabs(direction.y) && fabs(direction.x) > fabs(direction.z) ){
		if( direction.x < 0 ) this->drawYZ( intensity, ratio, cube );
		else                  this->drawZY( intensity, ratio, cube );
	}else if( fabs(direction.y) > fabs(direction.z) ){
		if( direction.y < 0 ) this->drawZX( intensity, ratio, cube );
		else                  this->drawXZ( intensity, ratio, cube );
	}else{
		if( direction.z < 0 ) this->drawXY( intensity, ratio, cube );
		else                  this->drawYX( intensity, ratio, cube );
	}
	og.PopMatrix();
}

void CCube::drawAtoms(){
	og.PushMatrix();
	og.Translate(-(origin+center));
	for(int i=0; i<abs(IFlag); i++){
		ogVector position = atom[i].position;
		switch( atom[i].number ){
			case  1:og.DrawSphere(position, 0.3, 255,255,255, 2); break;
			case  6:og.DrawSphere(position, 0.3, 255,0,0, 2); break;
			default:og.DrawSphere(position, 0.3, 0,0,255, 2); break;
		}
	}
	og.PopMatrix();
}

void CCube::drawAtoms( double ratio, CCube &cube ){
	og.PushMatrix();
	og.Translate(-(origin+center));
	for(int i=0; i<abs(IFlag); i++){
		ogVector position1 = atom[i].position*(1.0-ratio);
		ogVector position2 = cube.atom[i].position*ratio;
		ogVector position = position1 + position2;
		switch( atom[i].number ){
			case  1:og.DrawSphere(position, 0.3, 255,255,255, 2); break;
			case  6:og.DrawSphere(position, 0.3, 255,0,0, 2); break;
			default:og.DrawSphere(position, 0.3, 0,0,255, 2); break;
		}
	}
	og.PopMatrix();
}

void CCube::drawBonds(){
/*	double radius[31] = { 0.0, 0.03, 0.03, 0.134, 0.112, 0.082, 0.077, 0.074, 0.074, 0.071, 0.069,
			      0.154, 0.13, 0.118, 0.111, 0.106, 0.102, 0.099, 0.097, 0.196, 0.174,
			      0.144, 0.136, 0.125, 0.127, 0.139, 0.125, 0.126, 0.121, 0.138, 0.131};
*/
	double radius[119] = {
		999, 
		0.037, 0.032, 0.134, 0.090, 0.082, 0.077, 0.075, 0.073, 0.071, 0.069, 
		0.154, 0.130, 0.118, 0.111, 0.106, 0.102, 0.099, 0.097, 0.196, 0.174, 
		0.144, 0.136, 0.125, 0.127, 0.139, 0.125, 0.126, 0.121, 0.138, 0.131, 
		0.126, 0.122, 0.119, 0.116, 0.114, 0.110, 0.211, 0.192, 0.162, 0.148, 
		0.137, 0.145, 0.156, 0.126, 0.135, 0.131, 0.153, 0.148, 0.144, 0.145, 
		0.138, 0.135, 0.140, 0.130, 0.225, 0.198, 0.169, 999, 0.165, 999, 
		999, 999, 999, 999, 999, 999, 0.158, 999, 999, 999, 
		0.160, 0.150, 0.138, 0.135, 0.159, 0.128, 0.137, 0.128, 0.144, 0.149,  
		0.148, 0.147, 0.146, 0.999, 0.127, 0.145, 999, 999, 999, 999, 
		999, 999, 999, 999, 999, 999, 999, 999, 999, 999, 
		999, 999, 999, 999, 999, 999, 999, 999, 999, 999, 
		999, 999, 999, 999, 999, 999, 999, 999
	};

	for( int i=0; i<31; i++ ) radius[i] /= (0.0523167+0.007);

	og.PushMatrix();
	og.Translate(-(origin+center));
	for( int i=0; i<abs(IFlag); i++ ){	
		ogVector p1 = atom[i].position;
		for( int j=i+1; j<abs(IFlag); j++){
			ogVector p2 = atom[j].position;
			if( radius[atom[i].number] + radius[atom[j].number] + 0.5 > (p2-p1).abs() ){
				og.DrawCylinder(p1, p2, 0.1, 200, 200, 200, 16);
			}
		}
	}
	og.PopMatrix();
}

void CCube::drawBonds( double ratio, CCube &cube ){
/*	double radius[31] = { 0.0, 0.03, 0.03, 0.134, 0.112, 0.082, 0.077, 0.074, 0.074, 0.071, 0.069,
			      0.154, 0.13, 0.118, 0.111, 0.106, 0.102, 0.099, 0.097, 0.196, 0.174,
			      0.144, 0.136, 0.125, 0.127, 0.139, 0.125, 0.126, 0.121, 0.138, 0.131};
*/
	double radius[119] = {
		999, 
		0.037, 0.032, 0.134, 0.090, 0.082, 0.077, 0.075, 0.073, 0.071, 0.069, 
		0.154, 0.130, 0.118, 0.111, 0.106, 0.102, 0.099, 0.097, 0.196, 0.174, 
		0.144, 0.136, 0.125, 0.127, 0.139, 0.125, 0.126, 0.121, 0.138, 0.131, 
		0.126, 0.122, 0.119, 0.116, 0.114, 0.110, 0.211, 0.192, 0.162, 0.148, 
		0.137, 0.145, 0.156, 0.126, 0.135, 0.131, 0.153, 0.148, 0.144, 0.145, 
		0.138, 0.135, 0.140, 0.130, 0.225, 0.198, 0.169, 999, 0.165, 999, 
		999, 999, 999, 999, 999, 999, 0.158, 999, 999, 999, 
		0.160, 0.150, 0.138, 0.135, 0.159, 0.128, 0.137, 0.128, 0.144, 0.149,  
		0.148, 0.147, 0.146, 0.999, 0.127, 0.145, 999, 999, 999, 999, 
		999, 999, 999, 999, 999, 999, 999, 999, 999, 999, 
		999, 999, 999, 999, 999, 999, 999, 999, 999, 999, 
		999, 999, 999, 999, 999, 999, 999, 999
	};

	for( int i=0; i<31; i++ ) radius[i] /= (0.0523167+0.007);

	og.PushMatrix();
	og.Translate(-(origin+center));
	for( int i=0; i<abs(IFlag); i++ ){	
		for( int j=i+1; j<abs(IFlag); j++){
			ogVector p1 = atom[i].position; // p1 must be refreshed every time
			ogVector p2 = atom[j].position;
			if( radius[atom[i].number] + radius[atom[j].number] + 0.5 > (p2-p1).abs() ){
				p1 *= 1.0-ratio;
//				p1 += cube.atom[i].position*ratio;
				ogVector pp1 = cube.atom[i].position*ratio;
				p1 += pp1;
				p2 *= 1.0-ratio;
//				p2 += cube.atom[j].position*ratio;
				ogVector pp2 = cube.atom[j].position*ratio;
				p2 += pp2;
				//p1 -= origin + center;
				//p2 -= origin + center;
				og.DrawCylinder(p1, p2, 0.1, 200, 200, 200, 16);
			}
		}
	}
	og.PopMatrix();
}

void CCube::drawFrame( GLubyte red, GLubyte green, GLubyte blue ){
	ogVector v[8];
	ogVector xu = xUnit * double(xSize-1)/2.0;
	ogVector yu = yUnit * double(ySize-1)/2.0;
	ogVector zu = zUnit * double(zSize-1)/2.0;
	
	v[0] = ( - xu - yu - zu ); 	v[1] = ( - xu + yu - zu ); 	v[2] = (  xu + yu - zu );
	v[3] = (  xu - yu - zu ); 	v[4] = ( - xu - yu + zu ); 	v[5] = ( - xu + yu + zu );
	v[6] = (  xu + yu + zu ); 	v[7] = (  xu - yu + zu );
//	v[0] = ( - xUnit * double(xSize-1)/2.0 - yUnit * double(ySize-1)/2.0 - zUnit * double(zSize-1)/2.0 );
//	v[1] = ( - xUnit * double(xSize-1)/2.0 + yUnit * double(ySize-1)/2.0 - zUnit * double(zSize-1)/2.0 );
//	v[2] = (   xUnit * double(xSize-1)/2.0 + yUnit * double(ySize-1)/2.0 - zUnit * double(zSize-1)/2.0 );
//	v[3] = (   xUnit * double(xSize-1)/2.0 - yUnit * double(ySize-1)/2.0 - zUnit * double(zSize-1)/2.0 );
//	v[4] = ( - xUnit * double(xSize-1)/2.0 - yUnit * double(ySize-1)/2.0 + zUnit * double(zSize-1)/2.0 );
//	v[5] = ( - xUnit * double(xSize-1)/2.0 + yUnit * double(ySize-1)/2.0 + zUnit * double(zSize-1)/2.0 );
//	v[6] = (   xUnit * double(xSize-1)/2.0 + yUnit * double(ySize-1)/2.0 + zUnit * double(zSize-1)/2.0 );
//	v[7] = (   xUnit * double(xSize-1)/2.0 - yUnit * double(ySize-1)/2.0 + zUnit * double(zSize-1)/2.0 );
	for(int i=0; i<4; i++){
		og.DrawLine(v[i],v[(i+1)%4],red,green,blue);
		og.DrawLine(v[i+4],v[(i+1)%4+4],red,green,blue);
		og.DrawLine(v[i],v[i+4],red,green,blue);
	}
}

int CCube::loadCube( char *filename ){
FILE *fp;
char ch;
	if( (fp = fopen( filename, "r" )) == NULL ){
		printf("%s not found!n", filename);
		return 1;
	}
	while( (ch=fgetc(fp)) != '\n' );
	while( (ch=fgetc(fp)) != '\n' );

	fscanf( fp, "%d %lf %lf %lf", &IFlag, &origin.x, &origin.y, &origin.z );
	fscanf( fp, "%d %lf %lf %lf", &xSize, &xUnit.x, &xUnit.y, &xUnit.z );
	fscanf( fp, "%d %lf %lf %lf", &ySize, &yUnit.x, &yUnit.y, &yUnit.z );
	fscanf( fp, "%d %lf %lf %lf", &zSize, &zUnit.x, &zUnit.y, &zUnit.z );
	if( atom != NULL ) delete atom;
	atom = new CAtom[abs(IFlag)];
	for( long i=0; i< abs(IFlag); i++ ) atom[i].loadCube( fp );
	if( IFlag < 0 ) fscanf( fp, "%d %d", &num1, &num2 );
	if( d != NULL ) delete d;
	d = new float[xSize*ySize*zSize];

//	for( int i=0; i<xSize*ySize*zSize; i++ ) fscanf( fp, "%e", &d[i] );
	float ma=0,mi=50000;
	for( long i=0; i<xSize*ySize*zSize; i++ ) {
		fscanf( fp, "%e", &d[i] );
		if(d[i]<mi)mi=d[i];
		if(d[i]>ma)ma=d[i];
	}
	printf("max=%f,min=%f",ma,mi);
	maxv=ma; minv=mi;

	fclose( fp );
	printf( "%s loaded\n", filename );
	return 0;
}

int CCube::saveCube( char *filename ){
FILE *fp;
	if( (fp = fopen( filename, "w" )) == NULL ) return 1;
	fprintf( fp, "\n" );
	fprintf( fp, "\n" );

	fprintf( fp, "%d %lf %lf %lf\n", IFlag, origin.x, origin.y, origin.z );
	fprintf( fp, "%d %lf %lf %lf\n", xSize, xUnit.x, xUnit.y, xUnit.z );
	fprintf( fp, "%d %lf %lf %lf\n", ySize, yUnit.x, yUnit.y, yUnit.z );
	fprintf( fp, "%d %lf %lf %lf\n", zSize, zUnit.x, zUnit.y, zUnit.z );
	for( long i=0; i< abs(IFlag); i++ ) atom[i].saveCube( fp );
	if( IFlag < 0 ) fprintf( fp, "%d %d\n", num1, num2 );
	for( int i=0; i< xSize*ySize*zSize; i++ ) fprintf( fp, "%e ", d[i] );
	fclose( fp );
	return 0;
}

int CCube::loadBin( char *filename ){
FILE *fp;
	if( (fp = fopen( filename, "rb" )) == NULL ){
		printf("%s not found!n", filename);
		return 1;
	}
	fread( (int *)&IFlag, 1, sizeof(int), fp );
	fread( (double *)&origin.x, 1, sizeof(double), fp );
	fread( (double *)&origin.y, 1, sizeof(double), fp );
	fread( (double *)&origin.z, 1, sizeof(double), fp );
	fread( (int *)&xSize, 1, sizeof(int), fp );
	fread( (double *)&xUnit.x, 1, sizeof(double), fp );
	fread( (double *)&xUnit.y, 1, sizeof(double), fp );
	fread( (double *)&xUnit.z, 1, sizeof(double), fp );
	fread( (int *)&ySize, 1, sizeof(int), fp );
	fread( (double *)&yUnit.x, 1, sizeof(double), fp );
	fread( (double *)&yUnit.y, 1, sizeof(double), fp );
	fread( (double *)&yUnit.z, 1, sizeof(double), fp );
	fread( (int *)&zSize, 1, sizeof(int), fp );
	fread( (double *)&zUnit.x, 1, sizeof(double), fp );
	fread( (double *)&zUnit.y, 1, sizeof(double), fp );
	fread( (double *)&zUnit.z, 1, sizeof(double), fp );
	if( atom != NULL ) delete atom;
	atom = new CAtom[abs(IFlag)];
	for( int i=0; i< abs(IFlag); i++ ) atom[i].loadBin( fp );
	if( IFlag < 0 ){
		fread( (int *)&num1, 1, sizeof(int), fp );
		fread( (int *)&num2, 1, sizeof(int), fp );
	}
	if( d != NULL ) delete d;
	d = new float[xSize*ySize*zSize];
	fread( (float *)d, xSize*ySize*zSize, sizeof(float), fp );
	fclose( fp );

	float ma=0,mi=50000;
	int j;
	for( j=0; j<xSize*ySize*zSize; j++ ) {
		if(d[j]<mi)mi=d[j];
		if(d[j]>ma)ma=d[j];
	}
	printf("max=%f,min=%f",ma,mi);
	maxv=ma; minv=mi;

	return 0;
}

int CCube::saveBin( char *filename ){
FILE *fp;
	if( (fp = fopen( filename, "wb" )) == NULL ) return 1;
	fwrite( (int *)&IFlag, 1, sizeof(int), fp );
	fwrite( (double *)&origin.x, 1, sizeof(double), fp );
	fwrite( (double *)&origin.y, 1, sizeof(double), fp );
	fwrite( (double *)&origin.z, 1, sizeof(double), fp );
	fwrite( (int *)&xSize, 1, sizeof(int), fp );
	fwrite( (double *)&xUnit.x, 1, sizeof(double), fp );
	fwrite( (double *)&xUnit.y, 1, sizeof(double), fp );
	fwrite( (double *)&xUnit.z, 1, sizeof(double), fp );
	fwrite( (int *)&ySize, 1, sizeof(int), fp );
	fwrite( (double *)&yUnit.x, 1, sizeof(double), fp );
	fwrite( (double *)&yUnit.y, 1, sizeof(double), fp );
	fwrite( (double *)&yUnit.z, 1, sizeof(double), fp );
	fwrite( (int *)&zSize, 1, sizeof(int), fp );
	fwrite( (double *)&zUnit.x, 1, sizeof(double), fp );
	fwrite( (double *)&zUnit.y, 1, sizeof(double), fp );
	fwrite( (double *)&zUnit.z, 1, sizeof(double), fp );
	for( int i=0; i< abs(IFlag); i++ ) atom[i].saveBin( fp );
	if( IFlag < 0 ){
		fwrite( (int *)&num1, 1, sizeof(int), fp );
		fwrite( (int *)&num2, 1, sizeof(int), fp );
	}
	fwrite( (float *)d, xSize*ySize*zSize, sizeof(float), fp );
	fclose( fp );
	return 0;
}

int CCube::load( char *filename ){
	int i=0,j=0,k=0;
	while( filename[i] !='\0' ) i++;
	while( filename[i] !='.' ) i--;
	j=i;
	while( filename[j] !='\\' ) j--;
	while( filename[j+1] !='.' ) {
		filename0[k]=filename[j+1];
		j++;
		k++;
	}
	filename0[k]='.';	filename0[k+1]='m';	filename0[k+2]='t';	filename0[k+3]='i';	filename0[k+4]='\0';

	int result;
	if( filename[i+1] == 'c' ) result = loadCube( filename );
	else                       result = loadBin( filename );
	if( !result ) {
		ogVector xu = xUnit * double(xSize-1);
		ogVector yu = yUnit * double(ySize-1);
		ogVector zu = zUnit * double(zSize-1);
		center = xu + yu + zu;
		center *= 0.5;
//		center = (xUnit * double(xSize-1) + yUnit * double(ySize-1) + zUnit * double(zSize-1)) * 0.5;
	}
	return result;
}

int CCube::save( char *filename ){
	int i=0;
	while( filename[i] !='\0' ) i++;
	while( filename[i] !='.' ) i--;
	if( filename[i+1] == 'c' ) return saveCube( filename );
	else                       return saveBin( filename );
}


class ISODATA{
public:
	ogVector isoPosition;
	ogVector isoNormal;
	ISODATA *nextiso;
	int kind;
	int color;
};

class CCube_ISO:public CCube{
public:
ISODATA *iso_first;

int countValue;
int countall;
int counthalf;

void NearValueCheck(GLubyte value);
void NearValueCheck8(int value);

void DrawSurface(int drawtype, int pointsize,int ISOPOINT);
void DrawSurface(int drawtype, int pointsize, double rto, double *crto, double Its, double *cIts);
void DrawSurface(int drawtype, int pointsize, CCube_ISO &next_Cube,CCube_ISO *InCube, double rto, double *crto, double Its, double *cIts);
void drawpoint(int value, int ps);
void drawpoint(double rto, CCube_ISO &ci, int value, int ps);
void CCube_ISO::DrawOrbit(int drawtype, double ratio, double Intensity,
			  ogVector viewdirection, double *crto, double *cIts,
			  int pointsize, int IM);
void CCube_ISO::DrawOrbit(int drawtype, CCube_ISO &next_Cube, double ratio, double Intensity,
			  ogVector viewdirection, CCube_ISO *InCube, double *crto, double *cIts,
			  int pointsize, int IM);
};

void CCube_ISO::DrawSurface(int drawtype, int pointsize, double rto, double *crto, double Its, double *cIts){
	if(rto!=(*crto) || drawtype!=draw_mode){
		this->NearValueCheck8((int)(Its*255.0));
		*crto=rto;
		*cIts=Its;
		draw_mode=drawtype;
	}	
	if(Its!=(*cIts)){
		this->NearValueCheck8((int)(Its*255.0));
		*cIts=Its;
	}
	this->DrawSurface(drawtype,pointsize,0);
}

void CCube_ISO::DrawSurface(int drawtype, int pointsize, CCube_ISO &next_Cube,CCube_ISO *InCube, double rto, double *crto, double Its, double *cIts){
	if(rto!=(*crto) || draw_mode!=drawtype){
		draw_mode=drawtype;
		if(InCube->d!=NULL){delete InCube->d;};

		InCube->d = new float[xSize*ySize*zSize];
		InCube->xSize=this->xSize;
		InCube->ySize=this->ySize;
		InCube->zSize=this->zSize;
		InCube->xUnit=this->xUnit;
		InCube->yUnit=this->yUnit;
		InCube->zUnit=this->zUnit;

		for(int i=0;i<xSize*ySize*zSize;i++){
			InCube->d[i]=this->d[i]*(1-rto)+next_Cube.d[i]*rto;
		}
		InCube->NearValueCheck8((int)(Its*255.0));
		*crto=rto;
		*cIts=Its;
	}	
	if(Its!=(*cIts)){
//		InCube->NearValueCheck((int)(Its*255.0));
		InCube->NearValueCheck8((int)(Its*255.0));
		*cIts=Its;
	}
	InCube->DrawSurface(drawtype,pointsize,0);
}

void CCube_ISO::NearValueCheck8(int value){
//	printf("NearValueCheck8 s\n");
	int d[8];
	float d0[8];
	ogVector sv[8];
	ogVector svnormal[8];
	double percentage;
	double bunbo;
	ISODATA *isodata=NULL;


	int counter=0,counter2=0;
	int checkCube[8][3]={{1,3,4},{0,2,5},{1,3,6},{0,2,7},{0,5,7},{1,4,6},{2,5,7},{3,4,6}};
	int checkCube2[8]={6,7,4,5,2,3,0,1};

	int RS[2]={0,0};
	int MCP;
	int MCPnum[15]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

	int countp=0;
	int countm=0;
	int pmcheck[8]={0,0,0,0,0,0,0,0};

	countValue=0;
	countall=0;
	counthalf=0;

	isodata=(ISODATA*)malloc(sizeof(ISODATA)*2);
	iso_first=isodata;
for(int sec=0;sec<2;sec++){counthalf=countall;
	for( int x=0; x<xSize-1; x++ ){
		for( int z=0; z<zSize-1; z++ ){
			for( int y=0; y<ySize-1; y++ ){
				int point[2][8]={{-1,-1,-1,-1,-1,-1,-1,-1},{-1,-1,-1,-1,-1,-1,-1,-1}};

//				d[0] = cramp( int(this->density( x, y, z )) );
				
				d0[0] = this->density( x, y, z );
				sv[0].set(double(x), double(y), double(z));
				d0[1] = this->density( x+1, y, z );
				sv[1].set(double(x+1), double(y), double(z));
				d0[2] = this->density( x+1, y, z+1 );
				sv[2].set(double(x+1), double(y), double(z+1));
				d0[3] = this->density( x, y, z+1 );
				sv[3].set(double(x), double(y), double(z+1));

				d0[4] = this->density( x, y+1, z );
				sv[4].set(double(x), double(y+1), double(z));
				d0[5] = this->density( x+1, y+1, z );
				sv[5].set(double(x+1), double(y+1), double(z));
				d0[6] = this->density( x+1, y+1, z+1 );
				sv[6].set(double(x+1), double(y+1), double(z+1));
				d0[7] = this->density( x, y+1, z+1 );
				sv[7].set(double(x), double(y+1), double(z+1));

				//̔
				for(int pm=0;pm<8;pm++){
					if(d0[pm]<0) d[pm] = -cramp( int(-d0[pm]) );
					else d[pm] = cramp( int(d0[pm]) );
				}

				int i;
				counter=0; counter2=0;
				for(i=0;i<8;i++){
					if(value<d[i]){
						point[0][counter]=i;
						counter++;
					}
					else{
						point[1][counter2]=i;
						counter2++;
					}
				}

				if(counter>4){
					counter=counter2;
					for(i=0;i<8;i++){
						point[0][i]=point[1][i];
						point[1][i]=0;
					}
				}

				switch(counter){
					case 1:
						//r

						for(i=0;i<3;i++){
							if(d[point[0][0]]>d[checkCube[point[0][0]][i]]){
								percentage=(double)(value-d[checkCube[point[0][0]][i]]);
								bunbo=(double)(d[point[0][0]]-d[checkCube[point[0][0]][i]]);
								ogVector pa = sv[point[0][0]]*percentage;
								ogVector pb = sv[checkCube[point[0][0]][i]]*(bunbo-percentage);
								pa = pa + pb;
								pa = pa/bunbo; 
								isodata->isoPosition = pa;
							}
							else{
								percentage=(double)(value-d[point[0][0]]);
								bunbo=(double)(d[checkCube[point[0][0]][i]]-d[point[0][0]]);
								ogVector pa = sv[point[0][0]]*(bunbo-percentage);
								ogVector pb = sv[checkCube[point[0][0]][i]]*percentage;
								pa = pa + pb;
								pa = pa/bunbo; 
								isodata->isoPosition = pa;
							}
							isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
							isodata->kind=1;
							isodata=isodata->nextiso;
							countValue++;
						}
						MCPnum[1]++;
						break;
					case 2:
						MCP=checkM2(point[0],RS,sv,d);
						if(MCP==-1){rotationZ(point[0],sv,d);MCP=checkM2(point[0],RS,sv,d);}
						if(MCP==-1){rotationZ(point[0],sv,d);MCP=checkM2(point[0],RS,sv,d);}
						if(MCP==-1){rotationZ(point[0],sv,d);MCP=checkM2(point[0],RS,sv,d);}
						MCPnum[MCP]++;
						switch(MCP){
							case 2:
								isodata->isoPosition=setMC(sv,d,value,4,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,4,0);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,5,1);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,4,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,5,1);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,5,6);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;
								break;
							case 3:
								isodata->isoPosition=setMC(sv,d,value,4,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,4,0);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,4,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,1);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,1,2);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,1,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;
								//  ǉ
								isodata->isoPosition=setMC(sv,d,value,0,1);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata=isodata->nextiso;
								countValue++;
								isodata->isoPosition=setMC(sv,d,value,1,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata=isodata->nextiso;
								countValue++;
								isodata->isoPosition=setMC(sv,d,value,4,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata=isodata->nextiso;
								countValue++;
								isodata->isoPosition=setMC(sv,d,value,1,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata=isodata->nextiso;
								countValue++;
								isodata->isoPosition=setMC(sv,d,value,4,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata=isodata->nextiso;
								countValue++;
								isodata->isoPosition=setMC(sv,d,value,4,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata=isodata->nextiso;
								countValue++;
								isodata->isoPosition=setMC(sv,d,value,0,1);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata=isodata->nextiso;
								countValue++;
								isodata->isoPosition=setMC(sv,d,value,4,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata=isodata->nextiso;
								countValue++;
								isodata->isoPosition=setMC(sv,d,value,4,0);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata=isodata->nextiso;
								countValue++;
								break;
							case 4:
								isodata->isoPosition=setMC(sv,d,value,4,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,4,0);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,4,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,2,3);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,2,1);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,2,6);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;
								break;
						}
						break;
					case 3:
						MCP=checkM3(point[0],RS,sv,d);
						if(MCP==-1){rotationZ(point[0],sv,d);MCP=checkM3(point[0],RS,sv,d);}
						if(MCP==-1){rotationZ(point[0],sv,d);MCP=checkM3(point[0],RS,sv,d);}
						if(MCP==-1){rotationZ(point[0],sv,d);MCP=checkM3(point[0],RS,sv,d);}
						MCPnum[MCP]++;
						switch(MCP){
							case 5:
								isodata->isoPosition=setMC(sv,d,value,4,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,3);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,5,6);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,3);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,1,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,5,6);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,3);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,1);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,1,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;
								//ǉ
/*								isodata->isoPosition=setMC(sv,d,value,0,3);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata=isodata->nextiso;
								countValue++;
								isodata->isoPosition=setMC(sv,d,value,0,1);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata=isodata->nextiso;
								countValue++;
								isodata->isoPosition=setMC(sv,d,value,4,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata=isodata->nextiso;
								countValue++;
								isodata->isoPosition=setMC(sv,d,value,4,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata=isodata->nextiso;
								countValue++;
								isodata->isoPosition=setMC(sv,d,value,0,1);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata=isodata->nextiso;
								countValue++;
								isodata->isoPosition=setMC(sv,d,value,1,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata=isodata->nextiso;
								countValue++;
								isodata->isoPosition=setMC(sv,d,value,4,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata=isodata->nextiso;
								countValue++;
								isodata->isoPosition=setMC(sv,d,value,1,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata=isodata->nextiso;
								countValue++;
								isodata->isoPosition=setMC(sv,d,value,5,6);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata=isodata->nextiso;
								countValue++;
*/								break;
							case 6:
								isodata->isoPosition=setMC(sv,d,value,4,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,4,0);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,5,1);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,4,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,5,1);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,5,6);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,1,2);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,2,6);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,2,3);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;
								break;
							case 7:
								isodata->isoPosition=setMC(sv,d,value,4,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,4,0);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,4,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,1);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,1,2);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,1,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,5,6);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,2,6);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,6,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;
								break;
						}
						break;
					case 4:
						MCP=-1;
						MCP=checkM4(point[0],RS,sv,d);
						if(MCP==-1){rotationZ(point[0],sv,d);MCP=checkM4(point[0],RS,sv,d);}
						if(MCP==-1){rotationZ(point[0],sv,d);MCP=checkM4(point[0],RS,sv,d);}
						if(MCP==-1){rotationZ(point[0],sv,d);MCP=checkM4(point[0],RS,sv,d);}
						MCPnum[MCP]++;
						switch(MCP){
							case 8:
								isodata->isoPosition=setMC(sv,d,value,3,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,4);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,1,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,3,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,1,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,2,6);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;
								break;
							case 9:
								isodata->isoPosition=setMC(sv,d,value,3,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,3);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,6,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,6,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,3);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,1,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,1,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,3);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,1);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,5,6);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,6,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,1,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;
								break;
							case 10:
								isodata->isoPosition=setMC(sv,d,value,4,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,3);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,1);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,4,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,1);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,4,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,6,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,2,3);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,1,2);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,6,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,1,2);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,5,6);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;
								break;
							case 11:
								isodata->isoPosition=setMC(sv,d,value,4,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,4);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,5,6);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,5,6);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,4);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,2,6);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,2,6);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,4);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,1);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,2,6);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,1);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,2,3);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;
								break;
							case 12:
								isodata->isoPosition=setMC(sv,d,value,4,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,4);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,4,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,3);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,1,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,3,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,3);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,1);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,1,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,3,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,1,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,2,6);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;
								break;
							case 13:
								isodata->isoPosition=setMC(sv,d,value,4,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,4);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,4,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,1);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,1,2);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,1,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,5,6);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,2,6);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,6,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,3,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,2,3);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,3);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;
								break;
							case 14:
								isodata->isoPosition=setMC(sv,d,value,4,7);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,3);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,4,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,4,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,3);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,1,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,1,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,0,3);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,2,3);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,2,3);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,2,6);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;

								isodata->isoPosition=setMC(sv,d,value,1,5);
								isodata->nextiso=(ISODATA*)malloc(sizeof(ISODATA));
								isodata->kind=MCP;
								isodata=isodata->nextiso;
								countValue++;
								break;
						}
						break;
				}
				countall+=countValue;
				countValue=0;
			}
		}
	}
value=-value;
}
	sv[0].set(-1, 0, 0);
	sv[1].set( 1, 0, 0);
	sv[2].set( 0,-1, 0);
	sv[3].set( 0, 1, 0);
	sv[4].set( 0, 0,-1);
	sv[5].set( 0, 0, 1);//printf("x=%d,y=%d,z=%d",xSize,ySize,zSize);
	for(isodata=iso_first;countValue<countall;isodata=isodata->nextiso){//isodata->nextiso!=NULL
		double xd=isodata->isoPosition.x;
		double yd=isodata->isoPosition.y;
		double zd=isodata->isoPosition.z;
		int x = (int)(xd+0.5);
		int y = (int)(yd+0.5);
		int z = (int)(zd+0.5);
		if(xSize-1<=x){x=xSize-2;}
		if(ySize-1<=y){y=ySize-2;}
		if(zSize-1<=z){z=zSize-2;}
		if(0>=x){x=1;}
		if(0>=y){y=1;}
		if(0>=z){z=1;}
//		d[0] = cramp( int(this->density( x-1,   y,   z )) );//printf("x=%d,y=%d,z=%d",x,y,z);
//		d[1] = cramp( int(this->density( x+1,   y,   z )) );
//		d[2] = cramp( int(this->density(   x, y-1,   z )) );
//		d[3] = cramp( int(this->density(   x, y+1,   z )) );
//		d[4] = cramp( int(this->density(   x,   y, z-1 )) );
//		d[5] = cramp( int(this->density(   x,   y, z+1 )) );
//		d[6] = cramp( int(this->density( x, y, z )) );
		isodata->isoNormal.set(0.0,0.0,0.0);

		d0[0] =this->density( x-1,   y,   z );//printf("x=%d,y=%d,z=%d",x,y,z);
		d0[1] =this->density( x+1,   y,   z );
		d0[2] =this->density(   x, y-1,   z );
		d0[3] =this->density(   x, y+1,   z );
		d0[4] =this->density(   x,   y, z-1 );
		d0[5] =this->density(   x,   y, z+1 );
		d0[6] =this->density( x, y, z );
		for(int k=0; k<6;k++){
			if(d0[k]>0) d[k] = cramp( int(d0[k]) );
			else d[k] = cramp( int(-d0[k]) );
			if(d0[k]*d0[6]<0)d[k]=0;
		}


/*		if(this->density( x-1,   y,   z )<0)d[0]=-d[0];
		if(this->density( x+1,   y,   z )<0)d[1]=-d[1];
		if(this->density(   1, y-1,   z )<0)d[2]=-d[2];
		if(this->density(   1, y+1,   z )<0)d[3]=-d[3];
		if(this->density(   1,   y, z-1 )<0)d[4]=-d[4];
		if(this->density(   1,   y, z+1 )<0)d[5]=-d[5];
		if(this->density(   x,   y,   z )<0)d[6]=-d[6];
		sv[0].assign( x-1-xd, y-yd, z-zd);
		sv[1].assign( x+1-xd, y-yd, z-zd);
		sv[2].assign( x-xd,-1+y-yd, z-zd);
		sv[3].assign( x-xd, 1+y-yd, z-zd);
		sv[4].assign( x-xd, y-yd,-1+z-zd);
		sv[5].assign( x-xd, y-yd, 1+z-zd);
		d[6]=abs(value);
*/
		ogVector svn;
		for(int i=0;i<6;i++){
			svn = sv[i]*(double)(d[6]-d[i]);
			isodata->isoNormal+=svn;
		}

/*		if(countValue%3==0 || countValue==0){
		normalF=isodata;
		normalF->isoNormal=setNV(normalF->isoPosition,normalF->nextiso->isoPosition,normalF->nextiso->nextiso->isoPosition);
		if(normalF->vou==1){normalF->isoNormal=-normalF->isoNormal;}
		normalF->nextiso->isoNormal=normalF->isoNormal;
		normalF->nextiso->nextiso->isoNormal=normalF->isoNormal;
		}
*/
		isodata->isoNormal.normalize();

		if(counthalf>countValue)isodata->color=0;
		else isodata->color=1;
		
		//printf("(%lf,%lf,%lf)",isodata->isoNormal.x,isodata->isoNormal.y,isodata->isoNormal.z);
		countValue++;//printf("(%d)",countValue);
	}

}

void CCube_ISO::DrawSurface(int drawtype, int pointsize,int ISOPOINT){
//	printf("DrawSurface s\n");
	ogVector p[3],d,pp;
	ISODATA *isodata=NULL;

	ogVector xu = xUnit * double(xSize-1)/2.0;
	ogVector yu = yUnit * double(ySize-1)/2.0;
	ogVector zu = zUnit * double(zSize-1)/2.0;
	ogVector translate = -xu - yu - zu;
//	ogVector translate = - xUnit * double(xSize-1)/2.0 - yUnit * double(ySize-1)/2.0 - zUnit * double(zSize-1)/2.0;
	glPushMatrix();
	glTranslated( translate.x, translate.y, translate.z );


	glColor3d(1.0,0,0);

	if(drawtype==4){//C[t[
		og.ZBufferOn();
		if(alpha_on==1){
			glEnable(GL_BLEND);
//			glDepthMask(GL_FALSE);
			og.ZBufferOff();
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		}
		int endflag = 0;
		for(isodata=iso_first,countValue=0;countValue<countall;isodata=isodata->nextiso,countValue++){
			endflag = 0;
			if(countValue%3==0)	glBegin(GL_LINE_LOOP);
			int x=(int)(isodata->isoPosition.x);
			int y=(int)(isodata->isoPosition.y);
			int z=(int)(isodata->isoPosition.z);
			if(alpha_on==1){
				if(isodata->color==0)glColor4d(1.0,1.0,1.0,0.5);
				else glColor4d(1.0,0.0,0.0,0.4);
			}
			else{
				if(isodata->color==0)glColor3d(1.0,1.0,1.0);
				else glColor3d(1.0,0.0,0.0);
			}
			glNormal3d(isodata->isoNormal.x,isodata->isoNormal.y,isodata->isoNormal.z);
			glVertex3d(xUnit.abs()*isodata->isoPosition.x,yUnit.abs()*isodata->isoPosition.y,zUnit.abs()*isodata->isoPosition.z);
			if(countValue%3==2){
				glEnd();
				endflag = 1;
			}
		}
		if(endflag == 0)glEnd();
		if(alpha_on==1){
			og.ZBufferOn();
			glDisable(GL_BLEND);
		}
	}

	if(drawtype==3){//l
		og.ZBufferOn();
		if(alpha_on==1){
			glEnable(GL_BLEND);
//			glDepthMask(GL_FALSE);
			og.ZBufferOff();
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		}
		glBegin(GL_TRIANGLES);
		for(isodata=iso_first,countValue=0;countValue<countall;isodata=isodata->nextiso,countValue++){
			int x=(int)(isodata->isoPosition.x);
			int y=(int)(isodata->isoPosition.y);
			int z=(int)(isodata->isoPosition.z);
			if(alpha_on==1){
				if(isodata->color==0)glColor4d(1.0,1.0,1.0,0.5);
				else glColor4d(1.0,0.0,0.0,0.4);
			}
			else{
				if(isodata->color==0)glColor3d(1.0,1.0,1.0);
				else glColor3d(1.0,0.0,0.0);
			}
			glNormal3d(isodata->isoNormal.x,isodata->isoNormal.y,isodata->isoNormal.z);
			glVertex3d(xUnit.abs()*isodata->isoPosition.x,yUnit.abs()*isodata->isoPosition.y,zUnit.abs()*isodata->isoPosition.z);
		}
		glEnd();
		if(alpha_on==1){
			og.ZBufferOn();
			glDisable(GL_BLEND);
		}
	}
	glPopMatrix();
}

int convert_ftoi(float valuef){
	float maxd;
	if(maxv>fabsf(minv))maxd=maxv;
	else maxd=fabsf(minv);	
	return int(fabsf(valuef)*(255.0/maxd));
}

float convert_itof(int valuei){
	float maxd;
	if(maxv>fabsf(minv))maxd=maxv;
	else maxd=fabsf(minv);

	return float(fabsf(valuei)*(maxd/255.0));
}

int value_input(void){
	float iso_surface_value;
	printf("input value(%7.5f - %7.5f)\n>>",minv,maxv);
	scanf("%f",&iso_surface_value);
	
	return convert_ftoi(iso_surface_value);
}

void CCube_ISO::drawpoint(int value, int ps){
	int sizeZ,sizeYZ;
	int a,x,y,z;
	int di;
	float d[7], df;
	ogVector normal,sv[6];

	if(value<=0)value=1;
	sizeZ = this->zSize;
	sizeYZ = this->ySize * this->zSize;
	ogVector xu = this->xUnit * double(this->xSize-1)/2.0;
	ogVector yu = this->yUnit * double(this->ySize-1)/2.0;
	ogVector zu = this->zUnit * double(this->zSize-1)/2.0;
	ogVector translate = - xu - yu - zu;
	glPushMatrix();
	glTranslated( translate.x, translate.y, translate.z );

	glPointSize(ps);
	if(alpha_on==1){
		glEnable(GL_BLEND);
//		glDepthMask(GL_FALSE);
		glDisable(GL_DEPTH_TEST);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		
	}
	glBegin(GL_POINTS);

	for(int i=0;i<this->xSize*this->ySize*this->zSize-1;i++){
		df = this->d[i];
//		di = convert_ftoi(df);
//		df > 0 ? di = convert_ftoi(df) : di = convert_ftoi(-df);
		df > 0 ? di = cramp(int(df)) : di = cramp(int(-df));
		if(value <= di && di - value <= 5){
			a=i%sizeYZ;
			x=i/sizeYZ;
			y=a/sizeZ;
			z=a%sizeZ;

			if(x<=0 || y<=0 || z<=0)continue;
			if(x>=xSize-1 || y>=ySize-1 || z>=zSize-1)continue;

			d[0] = this->density( x-1,   y,   z );
			d[1] = this->density( x+1,   y,   z );
			d[2] = this->density(   x, y-1,   z );
			d[3] = this->density(   x, y+1,   z );
			d[4] = this->density(   x,   y, z-1 );
			d[5] = this->density(   x,   y, z+1 );
			d[6] = this->density( x, y, z );
			sv[0].set( -1,  0,  0 );
			sv[1].set(  1,  0,  0 );
			sv[2].set(  0, -1,  0 );
			sv[3].set(  0,  1,  0 );
			sv[4].set(  0,  0, -1 );
			sv[5].set(  0,  0,  1 );
			normal.set(0.0,0.0,0.0);
			for(int i=0;i<6;i++){
				ogVector svn = sv[i]*(double)(d[6]-d[i]);
				normal+=svn;
			}
			normal.normalize();

			if(alpha_on==1){
				if(this->density( x,y,z ) < 0)	glColor4d(1.0,0.0,0.0,0.05);
				else glColor4d(1.0,1.0,1.0,0.05);
			}
			else{
				if(this->density( x,y,z ) < 0)	glColor3d(1.0,0.0,0.0);
				else glColor3d(1.0,1.0,1.0);
			}
			glNormal3d(normal.x,normal.y,normal.z);
			glVertex3d(this->xUnit.abs()*x,this->yUnit.abs()*y,this->zUnit.abs()*z);
		}
	}
	glEnd();
	if(alpha_on==1){
		glEnable(GL_DEPTH_TEST);
		glDisable(GL_BLEND);
	}
	glPopMatrix();	
}


void CCube_ISO::drawpoint(double rto, CCube_ISO &ci, int value, int ps){
	int sizeZ,sizeYZ;
	int a,x,y,z;
	int di;
	float d[7], df;
	ogVector normal,sv[6];

	if(value<=0)value=1;
	sizeZ = this->zSize;
	sizeYZ = this->ySize * this->zSize;
	ogVector xu = this->xUnit * double(this->xSize-1)/2.0;
	ogVector yu = this->yUnit * double(this->ySize-1)/2.0;
	ogVector zu = this->zUnit * double(this->zSize-1)/2.0;
	ogVector translate = - xu - yu - zu;
	glPushMatrix();
	glTranslated( translate.x, translate.y, translate.z );

	glPointSize(ps);
	if(alpha_on==1){
		glEnable(GL_BLEND);
//		glDepthMask(GL_FALSE);
		glDisable(GL_DEPTH_TEST);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		
	}
	glBegin(GL_POINTS);

	for(int i=0;i<this->xSize*this->ySize*this->zSize-1;i++){
		df = this->d[i]*(1.0-rto)+ci.d[i]*rto;
//		di = convert_ftoi(df/2.0);
//		df > 0 ? di = convert_ftoi(df) : di = convert_ftoi(-df);
		df > 0 ? di = cramp(int(df)) : di = cramp(int(-df));
		if(value <= di && di - value <= 5){
			a=i%sizeYZ;
			x=i/sizeYZ;
			y=a/sizeZ;
			z=a%sizeZ;

			if(x<=0 || y<=0 || z<=0)continue;
			if(x>=xSize-1 || y>=ySize-1 || z>=zSize-1)continue;

			d[0] = this->density( x-1,   y,   z );
			d[1] = this->density( x+1,   y,   z );
			d[2] = this->density(   x, y-1,   z );
			d[3] = this->density(   x, y+1,   z );
			d[4] = this->density(   x,   y, z-1 );
			d[5] = this->density(   x,   y, z+1 );
			d[6] = this->density( x, y, z );
			sv[0].set( -1,  0,  0 );
			sv[1].set(  1,  0,  0 );
			sv[2].set(  0, -1,  0 );
			sv[3].set(  0,  1,  0 );
			sv[4].set(  0,  0, -1 );
			sv[5].set(  0,  0,  1 );
			normal.set(0.0,0.0,0.0);
			for(int i=0;i<6;i++){
				ogVector svn = sv[i]*(double)(d[6]-d[i]);
				normal+=svn;
			}
			normal.normalize();

			if(alpha_on==1){
				if(this->density( x,y,z ) < 0)	glColor4d(1.0,0.0,0.0,0.05);
				else glColor4d(1.0,1.0,1.0,0.05);
			}
			else{
				if(this->density( x,y,z ) < 0)	glColor3d(1.0,0.0,0.0);
				else glColor3d(1.0,1.0,1.0);
			}
			glNormal3d(normal.x,normal.y,normal.z);
			glVertex3d(this->xUnit.abs()*x,this->yUnit.abs()*y,this->zUnit.abs()*z);
		}
	}
	glEnd();
	if(alpha_on==1){
		glEnable(GL_DEPTH_TEST);
		glDisable(GL_BLEND);
	}
	glPopMatrix();	
}
void CCube_ISO::DrawOrbit(int drawtype, double ratio, double Intensity,
			  ogVector viewdirection, double *crto, double *cIts,
			  int pointsize, int IM){

	switch(drawtype){
		case 0:
			d_type=TYPE_C;
			og.ZBufferOff();
			glEnable( GL_TEXTURE_2D );
			glEnable( GL_BLEND );
			this->draw( viewdirection, Intensity );
			glDisable( GL_BLEND );
			glDisable( GL_TEXTURE_2D );
			og.ZBufferOn();
			break;
		case 1:
			d_type=TYPE_I;
			og.ZBufferOff();
			glEnable( GL_TEXTURE_2D );
			glEnable( GL_BLEND );
			this->draw( viewdirection, Intensity );
			glDisable( GL_BLEND );
			glDisable( GL_TEXTURE_2D );
			og.ZBufferOn();
			break;
		case 2:
			og.ZBufferOn();
			glDisable( GL_TEXTURE_2D );
			glDisable( GL_BLEND );
			this->drawpoint(IM, pointsize);
			glEnable( GL_BLEND );
			glEnable( GL_TEXTURE_2D );
			og.ZBufferOff();
			break;
		case 3:
			og.ZBufferOn();
			glDisable( GL_TEXTURE_2D );
			glDisable( GL_BLEND );
			this->DrawSurface(drawtype, pointsize, ratio, crto, double(IM)/255.0, cIts);
			glEnable( GL_BLEND );
			glEnable( GL_TEXTURE_2D );
			og.ZBufferOff();
			break;
		case 4:
			og.ZBufferOn();
			glDisable( GL_TEXTURE_2D );
			glDisable( GL_BLEND );
			this->DrawSurface(drawtype, pointsize, ratio, crto, double(IM)/255.0, cIts);
			glEnable( GL_BLEND );
			glEnable( GL_TEXTURE_2D );
			og.ZBufferOff();
			break;
		default:
			break;
	}
}

void CCube_ISO::DrawOrbit(int drawtype, CCube_ISO &next_Cube, double ratio, double Intensity,
			  ogVector viewdirection, CCube_ISO *InCube, double *crto, double *cIts,
			  int pointsize, int IM){

	switch(drawtype){
		case 0:
			d_type=TYPE_C;
			og.ZBufferOff();
			glEnable( GL_TEXTURE_2D );
			glEnable( GL_BLEND );
			this->draw( viewdirection, Intensity, ratio, next_Cube );
			glDisable( GL_BLEND );
			glDisable( GL_TEXTURE_2D );
			og.ZBufferOn();
			break;
		case 1:
			d_type=TYPE_I;
			og.ZBufferOff();
			glEnable( GL_TEXTURE_2D );
			glEnable( GL_BLEND );
			this->draw( viewdirection, Intensity, ratio, next_Cube );
			glDisable( GL_BLEND );
			glDisable( GL_TEXTURE_2D );
			og.ZBufferOn();
			break;
		case 2:
			og.ZBufferOn();
			glDisable( GL_TEXTURE_2D );
			glDisable( GL_BLEND );
			this->drawpoint(ratio, next_Cube, IM, pointsize);
			glEnable( GL_BLEND );
			glEnable( GL_TEXTURE_2D );
			og.ZBufferOff();
			break;
		case 3:
			og.ZBufferOn();
			glDisable( GL_TEXTURE_2D );
			glDisable( GL_BLEND );
			this->DrawSurface(drawtype, pointsize, next_Cube, InCube, ratio, crto, double(IM)/255.0, cIts);
			glEnable( GL_BLEND );
			glEnable( GL_TEXTURE_2D );
			og.ZBufferOff();
			break;
		case 4:
			og.ZBufferOn();
			glDisable( GL_TEXTURE_2D );
			glDisable( GL_BLEND );
			this->DrawSurface(drawtype, pointsize, next_Cube, InCube, ratio, crto, double(IM)/255.0, cIts);
			glEnable( GL_BLEND );
			glEnable( GL_TEXTURE_2D );
			og.ZBufferOff();
			break;
		default:
			break;
	}
}